@@ -12,26 +12,22 @@ require(ETCDIR . '/judgehost-config.php');
 
 $waittime = 5;
 
-$myhost = trim(`hostname | cut -d . -f 1`);
-
 define ('SCRIPT_ID', 'judgedaemon');
-define ('LOGFILE', LOGDIR.'/judge.'.$myhost.'.log');
 define ('PIDFILE', RUNDIR.'/judgedaemon.pid');
 
-require(LIBDIR . '/init.php');
-
 function usage()
 {
 	echo "Usage: " . SCRIPT_ID . " [OPTION]...\n" .
 	    "Start the judgedaemon.\n\n" .
 	    "  -d       daemonize after startup\n" .
+	    "  -n <id>  daemon number\n" .
 	    "  -v       set verbosity to LEVEL (syslog levels)\n" .
 	    "  -h       display this help and exit\n" .
 	    "  -V       output version information and exit\n\n";
 	exit;
 }
 
-$options = getopt("dv:hV");
+$options = getopt("dv:n:hV");
 // With PHP version >= 5.3 we can also use long options.
 // FIXME: getopt doesn't return FALSE on parse failure as documented!
 if ( $options===FALSE ) {
@@ -40,20 +36,48 @@ if ( $options===FALSE ) {
 }
 if ( isset($options['d']) ) $options['daemon']  = $options['d'];
 if ( isset($options['v']) ) $options['verbose'] = $options['v'];
+if ( isset($options['n']) ) $options['daemonid'] = $options['n'];
 
 if ( isset($options['V']) ) version();
 if ( isset($options['h']) ) usage();
 
+$myhost = trim(`hostname | cut -d . -f 1`);
+if ( isset($options['daemonid']) ) {
+	if ( !defined('USE_CGROUPS') || !USE_CGROUPS ) {
+		echo "Option `-n' is only supported when compiled with cgroup support.\n";
+		exit(1);
+	}
+	if ( preg_match('/^\d+$/', $options['daemonid'] ) ) {
+		$myhost = $myhost . "-" . $options['daemonid'];	
+	} else {
+		echo "Invalid value for daemonid, must be positive integer.\n";
+		exit(1);
+	}
+}
+
+define ('LOGFILE', LOGDIR.'/judge.'.$myhost.'.log');
+require(LIBDIR . '/init.php');
+
 setup_database_connection();
 
 $verbose = LOG_INFO;
-if ( isset($options['verbose']) ) $verbose = $options['verbose'];
+if ( isset($options['verbose']) ) {
+	if ( preg_match('/^\d+$/', $options['verbose'] ) ) {
+		$verbose = $options['verbose'];	
+	} else {
+		echo "Invalid value for verbose, must be positive integer\n";
+		exit(1);
+	}
+}
 
 if ( DEBUG & DEBUG_JUDGE ) {
 	$verbose = LOG_DEBUG;
 	putenv('DEBUG=1');
 }
 
+$runuser = RUNUSER;
+if ( isset($options['daemonid']) ) $runuser .= '-' . $options['daemonid'];
+
 // Set static environment variables for passing path configuration
 // to called programs:
 putenv('DJ_BINDIR='      . BINDIR);
@@ -62,7 +86,7 @@ putenv('DJ_JUDGEDIR='    . JUDGEDIR);
 putenv('DJ_LIBDIR='      . LIBDIR);
 putenv('DJ_LIBJUDGEDIR=' . LIBJUDGEDIR);
 putenv('DJ_LOGDIR='      . LOGDIR);
-putenv('RUNUSER='        . RUNUSER);
+putenv('RUNUSER='        . $runuser);
 
 foreach ( $EXITCODES as $code => $name ) {
 	$var = 'E_' . strtoupper(str_replace('-','_',$name));
@@ -72,12 +96,12 @@ foreach ( $EXITCODES as $code => $name ) {
 // Pass SYSLOG variable via environment for compare program
 if ( defined('SYSLOG') && SYSLOG ) putenv('DJ_SYSLOG=' . SYSLOG);
 
-system("pgrep -u ".RUNUSER, $retval);
+system("pgrep -u $runuser", $retval);
 if ($retval == 0) {
-	error("Still some processes by ".RUNUSER." found, aborting");
+	error("Still some processes by $runuser found, aborting");
 }
 if ($retval != 1) {
-	error("Error while checking processes for user " . RUNUSER);
+	error("Error while checking processes for user $runuser");
 }
 
 logmsg(LOG_NOTICE, "Judge started on $myhost [DOMjudge/".DOMJUDGE_VERSION."]");
@@ -93,46 +117,13 @@ if ( isset($options['daemon']) ) daemonize(PIDFILE);
 
 database_retry_connect($waittime);
 
-$first = True;
-while( !$exitsignalled )
-{
-	try {
-		// Retrieve hostname and check database for judgehost entry
-		$row = $DB->q('MAYBETUPLE SELECT * FROM judgehost WHERE hostname = %s'
-		             , $myhost);
-		if ( ! $row ) {
-			if($first)
-				logmsg(LOG_WARNING, "No database entry found for me ($myhost)");
-			$first = False;
-			sleep($waittime);
-			continue;
-		}
-		$myhost = $row['hostname'];
-		unset($first);
-		break;
-	}
-	catch( Exception $e ) {
-		$msg = "MySQL server has gone away";
-		if( ! strncmp($e->getMessage(), $msg, strlen($msg)) ) {
-			logmsg(LOG_WARNING, $msg);
-			database_retry_connect();
-			continue;
-		}
-		throw $e;
-	}
-}
-
-// If there are any unfinished judgings in the queue in my name,
-// they will not be finished. Give them back.
-$res = $DB->q('SELECT judgingid, submitid FROM judging WHERE
-               judgehost = %s AND endtime IS NULL AND valid = 1', $myhost);
-while ( $jud = $res->next() ) {
-	$DB->q('UPDATE judging SET valid = 0 WHERE judgingid = %i',
-	       $jud['judgingid']);
-	$DB->q('UPDATE submission SET judgehost = NULL, judgemark = NULL
-	        WHERE submitid = %i', $jud['submitid']);
-	logmsg(LOG_WARNING, "Found unfinished judging j" . $jud['judgingid'] . " in my name; given back");
-	auditlog('judging', $jud['judgingid'], 'given back', null, $myhost);
+// Check database for judgehost entry
+$row = $DB->q('MAYBETUPLE SELECT * FROM judgehost WHERE hostname = %s'
+             , $myhost);
+if ( ! $row ) {
+	logmsg(LOG_INFO, "No database entry found for me ($myhost), registering");
+	$DB->q('INSERT INTO judgehost (hostname) VALUES (%s)'
+	      , $myhost);
 }
 
 // Warn when chroot has been disabled. This has security implications.
@@ -144,6 +135,22 @@ if ( ! USE_CHROOT ) {
 $workdirpath = JUDGEDIR . "/$myhost";
 system("mkdir -p $workdirpath/testcase", $retval);
 if ( $retval != 0 ) error("Could not create $workdirpath");
+chmod("$workdirpath/testcase", 0700);
+
+// If there are any unfinished judgings in the queue in my name,
+// they will not be finished. Give them back.
+$res = $DB->q('SELECT judgingid, submitid, cid FROM judging WHERE
+               judgehost = %s AND endtime IS NULL AND valid = 1', $myhost);
+while ( $jud = $res->next() ) {
+	$workdir = "$workdirpath/c$jud[cid]-s$jud[submitid]-j$jud[judgingid]";
+	@chmod($workdir, 0700);
+	$DB->q('UPDATE judging SET valid = 0 WHERE judgingid = %i',
+	       $jud['judgingid']);
+	$DB->q('UPDATE submission SET judgehost = NULL, judgemark = NULL
+	        WHERE submitid = %i', $jud['submitid']);
+	logmsg(LOG_WARNING, "Found unfinished judging j" . $jud['judgingid'] . " in my name; given back");
+	auditlog('judging', $jud['judgingid'], 'given back', null, $myhost);
+}
 
 $waiting = FALSE;
 $active = TRUE;
@@ -236,7 +243,7 @@ while ( TRUE ) {
 		                    WHERE judgemark IS NULL AND cid = %i
  		                    AND langid IN (%As) AND probid IN (%As)
  		                    AND submittime < %s AND valid = 1
-		                    ORDER BY judging_last_started ASC, submittime ASC
+		                    ORDER BY judging_last_started ASC, submittime ASC, submitid ASC
 		                    LIMIT 1',
 		                   $cid, $judgable_lang, $judgable_prob,
 		                   $cdata['endtime']);
@@ -298,7 +305,7 @@ while ( TRUE ) {
 
 function judge($mark, $row, $judgingid)
 {
-	global $EXITCODES, $DB, $cid, $myhost, $workdirpath;
+	global $EXITCODES, $DB, $cid, $myhost, $options, $workdirpath;
 
 	// Set configuration variables for called programs
 	// Call dbconfig_init() to prevent using cached values.
@@ -309,6 +316,9 @@ function judge($mark, $row, $judgingid)
 	putenv('FILELIMIT='     . dbconfig_get('filesize_limit'));
 	putenv('PROCLIMIT='     . dbconfig_get('process_limit'));
 
+	$cpuset_opt = "";
+	if ( isset($options['daemonid']) ) $cpuset_opt = "-n ${options['daemonid']}";
+
 	// create workdir for judging
 	$workdir = "$workdirpath/c$cid-s$row[submitid]-j$judgingid";
 
@@ -321,12 +331,15 @@ function judge($mark, $row, $judgingid)
 		if ( !rename($workdir, $oldworkdir) ) {
 			error("Could not rename stale working directory to '$oldworkdir'");
 		}
+		@chmod($oldworkdir, 0700);
 		warning("Found stale working directory; renamed to '$oldworkdir'");
 	}
 
 	system("mkdir -p '$workdir/compile'", $retval);
 	if ( $retval != 0 ) error("Could not create '$workdir/compile'");
 
+	if ( !chdir($workdir) ) error("Could not chdir to '$workdir'");
+
 	// Get the source code from the DB and store in local file(s)
 	$sources = $DB->q('KEYTABLE SELECT rank AS ARRAYKEY, sourcecode, filename
 	                   FROM submission_file WHERE submitid = %i', $row['submitid']);
@@ -340,7 +353,7 @@ function judge($mark, $row, $judgingid)
 	}
 
 	// Compile the program.
-	system(LIBJUDGEDIR . "/compile.sh $row[langid] '$workdir' " .
+	system(LIBJUDGEDIR . "/compile.sh $cpuset_opt $row[langid] '$workdir' " .
 	       implode(' ', $files), $retval);
 
 	// what does the exitcode mean?
@@ -356,7 +369,9 @@ function judge($mark, $row, $judgingid)
 
 	// Only continue running testcases when compilation was successful.
 	// FIXME(?): result is still returned as in EXITCODES.
-	if ( ($result = $EXITCODES[$retval])=='correct' ) {
+	if ( ($result = $EXITCODES[$retval])=='compiler-error' ) {
+		store_result($result, $row, $judgingid);
+	} else {
 
 	logmsg(LOG_DEBUG, "Fetching testcases from database");
 	$testcases = $DB->q("KEYTABLE SELECT rank AS ARRAYKEY,
@@ -372,11 +387,15 @@ function judge($mark, $row, $judgingid)
 	// Optionally create chroot environment
 	if ( USE_CHROOT && CHROOT_SCRIPT ) {
 		logmsg(LOG_INFO, "executing chroot script: '".CHROOT_SCRIPT." start'");
-		chdir($workdir);
 		system(LIBJUDGEDIR.'/'.CHROOT_SCRIPT.' start', $retval);
 		if ( $retval!=0 ) error("chroot script exited with exitcode $retval");
 	}
 
+	// Make sure the workdir is accessible for the domjudge-run user.	
+	// Will be revoked again after this run finished.
+	chmod ($workdir, 0755);
+
+	$final = FALSE;
 	foreach ( $testcases as $tc ) {
 
 	logmsg(LOG_DEBUG, "Running testcase $tc[rank]...");
@@ -384,6 +403,7 @@ function judge($mark, $row, $judgingid)
 
 	// Get both in- and output files, only if we didn't have them already.
 	$tcfile = array();
+	$fetched = array();
 	foreach(array('input','output') as $inout) {
 		$tcfile[$inout] = "$workdirpath/testcase/testcase.$tc[probid].$tc[rank]." .
 		    $tc['md5sum_'.$inout] . "." . substr($inout, 0, -3);
@@ -400,8 +420,7 @@ function judge($mark, $row, $judgingid)
 			} else {
 				error("File corrupted during download.");
 			}
-			logmsg(LOG_NOTICE, "Fetched new $inout testcase $tc[rank] for " .
-			       "problem $row[probid]");
+			$fetched[] = $inout;
 		}
 		// sanity check (NOTE: performance impact is negligible with 5
 		// testcases and total 3.3 MB of data)
@@ -409,6 +428,11 @@ function judge($mark, $row, $judgingid)
 			error("File corrupted: md5sum mismatch: " . $tcfile[$inout]);
 		}
 	}
+	// Only log downloading input and/or output testdata once.
+	if ( count($fetched)>0 ) {
+		logmsg(LOG_INFO, "Fetched new " . implode($fetched,',') .
+		       " testcase $tc[rank] for problem $row[probid]");
+	}
 
 	// Copy program with all possible additional files to testcase
 	// dir. Use hardlinks to preserve space with big executables.
@@ -420,7 +444,7 @@ function judge($mark, $row, $judgingid)
 	if ( $retval!=0 ) error("Could not copy program to '$programdir'");
 
 	// do the actual test-run
-	system(LIBJUDGEDIR . "/testcase_run.sh $tcfile[input] $tcfile[output] " .
+	system(LIBJUDGEDIR . "/testcase_run.sh $cpuset_opt $tcfile[input] $tcfile[output] " .
 	       "$row[maxruntime] '$testcasedir' " .
 	       "'$row[special_run]' '$row[special_compare]'", $retval);
 
@@ -456,49 +480,36 @@ function judge($mark, $row, $judgingid)
 
 	// Optimization: stop judging when the result is already known.
 	// This should report a final result when all runresults are non-null!
-	if ( ($result = getFinalResult($runresults))!==NULL ) break;
+	if ( !$final && ($result = getFinalResult($runresults))!==NULL ) {
+		$final = TRUE;
+
+		store_result($result, $row, $judgingid);
+
+		if ( dbconfig_get('lazy_eval_results', true) ) {
+			break;
+		}
+	}
 
 	} // end: for each testcase
 
+	// revoke readablity for domjudge-run user to this workdir
+	chmod($workdir, 0700);
+
 	// Optionally destroy chroot environment
 	if ( USE_CHROOT && CHROOT_SCRIPT ) {
 		logmsg(LOG_INFO, "executing chroot script: '".CHROOT_SCRIPT." stop'");
-		chdir($workdir);
 		system(LIBJUDGEDIR.'/'.CHROOT_SCRIPT.' stop', $retval);
 		if ( $retval!=0 ) error("chroot script exited with exitcode $retval");
 	}
 
-	} // end: if compile result==0
+	} // end: if no compile-error
 
 	if ( $result==NULL ) error("No final result obtained");
 
-	// Start a transaction. This will provide extra safety if the table type
-	// supports it.
-	$DB->q('START TRANSACTION');
-	// pop the result back into the judging table
-	$DB->q('UPDATE judging SET endtime = %s, result = %s
+	// Store judging endtime, result was already stored.
+	$DB->q('UPDATE judging SET endtime = %s
 	        WHERE judgingid = %i AND judgehost = %s',
-	       now(), $result, $judgingid, $myhost);
-
-	// recalculate the scoreboard cell (team,problem) after this judging
-	calcScoreRow($cid, $row['teamid'], $row['probid']);
-
-	// log to event table if no verification required
-	// (case of verification required is handled in www/jury/verify.php)
-	if ( ! dbconfig_get('verification_required', 0) ) {
-		$DB->q('INSERT INTO event (eventtime, cid, teamid, langid, probid,
-		                           submitid, judgingid, description)
-		        VALUES(%s, %i, %s, %s, %s, %i, %i, "problem judged")',
-		       now(), $cid, $row['teamid'], $row['langid'], $row['probid'],
-		       $row['submitid'], $judgingid);
-		if ( $result == 'correct' ) {
-			$DB->q('INSERT INTO balloon (submitid)
-			        VALUES(%i)',
-			        $row['submitid']);
-		} 
-	}
-
-	$DB->q('COMMIT');
+	       now(), $judgingid, $myhost);
 
 	// done!
 	logmsg(LOG_NOTICE, "Judging s$row[submitid]/j$judgingid finished, result: $result");
@@ -533,4 +544,48 @@ function database_retry_connect()
 			throw $e;
 		}
 	}
+
+	if ( $exitsignalled ) {
+		logmsg(LOG_NOTICE, "Received signal, exiting.");
+		exit;
+	}
+}
+
+function store_result($result, $row, $judgingid)
+{
+	global $DB, $cid, $myhost;
+
+	// Start a transaction. This will provide extra safety if the table type
+	// supports it.
+	$DB->q('START TRANSACTION');
+	// pop the result back into the judging table
+	$DB->q('UPDATE judging SET result = %s
+	        WHERE judgingid = %i AND judgehost = %s',
+	       $result, $judgingid, $myhost);
+
+	// recalculate the scoreboard cell (team,problem) after this judging
+	calcScoreRow($cid, $row['teamid'], $row['probid']);
+
+	// log to event table if no verification required
+	// (case of verification required is handled in www/jury/verify.php)
+	if ( ! dbconfig_get('verification_required', 0) ) {
+		$DB->q('INSERT INTO event (eventtime, cid, teamid, langid, probid,
+		                           submitid, judgingid, description)
+		        VALUES(%s, %i, %s, %s, %s, %i, %i, "problem judged")',
+		       now(), $cid, $row['teamid'], $row['langid'], $row['probid'],
+		       $row['submitid'], $judgingid);
+		if ( $result == 'correct' ) {
+			// prevent duplicate balloons in case of multiple correct submissions
+			$numcorrect = $DB->q('VALUE SELECT count(submitid)
+			                      FROM balloon LEFT JOIN submission USING(submitid)
+			                      WHERE valid = 1 AND probid = %s AND teamid = %s',
+			                      $row['probid'], $row['teamid']);
+			if ( $numcorrect == 0 ) {
+				$DB->q('INSERT INTO balloon (submitid) VALUES(%i)',
+				       $row['submitid']);
+			}
+		}
+	}
+
+	$DB->q('COMMIT');
 }
