@@ -1,6 +1,7 @@
 /*
    runguard -- run command with restrictions.
-   Copyright (C) 2004-2012 Jaap Eldering (eldering@a-eskwadraat.nl).
+   Copyright (C) 2004-2013 Jaap Eldering (eldering@a-eskwadraat.nl)
+   Copyright (C) 2013      Keith Johnson
 
    Multiple minor improvements ported from the DOMjudge-ETH tree.
 
@@ -39,8 +40,15 @@
 
 #include "config.h"
 
+/* Some system/site specific config: VALID_USERS, CHROOT_PREFIX */
+#include "runguard-config.h"
+
 /* For chroot(), which is not POSIX. */
 #define _BSD_SOURCE
+/* For unshare(), only used when cgroups are enabled */
+#if ( USE_CGROUPS == 1 )
+#define _GNU_SOURCE
+#endif
 
 #include <sys/types.h>
 #include <sys/wait.h>
@@ -64,9 +72,13 @@
 #include <time.h>
 #include <math.h>
 #include <limits.h>
-
-/* Some system/site specific config: VALID_USERS, CHROOT_PREFIX */
-#include "runguard-config.h"
+#if ( USE_CGROUPS == 1 )
+#include <inttypes.h>
+#include <libcgroup.h>
+#include <sched.h>
+#else
+#undef USE_CGROUPS
+#endif
 
 #define PROGRAM "runguard"
 #define VERSION DOMJUDGE_VERSION "/" REVISION
@@ -99,6 +111,10 @@ char  *stdoutfilename;
 char  *stderrfilename;
 char  *exitfilename;
 char  *timefilename;
+#ifdef USE_CGROUPS
+char  *cgroupname;
+const char *cpuset;
+#endif
 
 int runuid;
 int rungid;
@@ -118,9 +134,12 @@ int be_quiet;
 int show_help;
 int show_version;
 
-unsigned long runtime; /* in microseconds */
-unsigned long cputime;
+double runtime, cputime; /* in seconds */
+#ifdef USE_CGROUPS
+int64_t memsize;
+#else
 rlim_t memsize;
+#endif
 rlim_t filesize;
 rlim_t nproc;
 size_t streamsize;
@@ -146,6 +165,7 @@ struct option const long_opts[] = {
 	{"memsize",    required_argument, NULL,         'm'},
 	{"filesize",   required_argument, NULL,         'f'},
 	{"nproc",      required_argument, NULL,         'p'},
+	{"cpuset",     required_argument, NULL,         'P'},
 	{"no-core",    no_argument,       NULL,         'c'},
 	{"stdout",     required_argument, NULL,         'o'},
 	{"stderr",     required_argument, NULL,         'e'},
@@ -242,6 +262,7 @@ Run COMMAND with restrictions.\n\
   -f, --filesize=SIZE    set maximum created filesize to SIZE kB;\n");
 	printf("\
   -p, --nproc=N          set maximum no. processes to N\n\
+  -P, --cpuset=ID        use only processor number ID\n\
   -c, --no-core          disable core dumps\n\
   -o, --stdout=FILE      redirect COMMAND stdout output to FILE\n\
   -e, --stderr=FILE      redirect COMMAND stderr output to FILE\n\
@@ -288,7 +309,7 @@ void output_exit_time(int exitcode, double timediff)
 	verbose("runtime is %.3f seconds real, %.3f user, %.3f sys",
 	        timediff, userdiff, sysdiff);
 
-	if ( use_cputime && (userdiff+sysdiff) * 1000000 > cputime ) {
+	if ( use_cputime && (userdiff+sysdiff) > cputime ) {
 		warning("timelimit exceeded (cpu time)");
 	}
 
@@ -307,6 +328,117 @@ void output_exit_time(int exitcode, double timediff)
 	}
 }
 
+#ifdef USE_CGROUPS
+void output_cgroup_stats()
+{
+	int ret;
+	int64_t max_usage;
+	struct cgroup *cg;
+	struct cgroup_controller *cg_controller;
+
+	cg = cgroup_new_cgroup(cgroupname);
+	if (!cg) {
+		error(0,"cgroup_new_cgroup");
+	}
+	if ((ret = cgroup_get_cgroup(cg)) != 0) {
+		error(0,"get cgroup information - %s(%d)", cgroup_strerror(ret), ret);
+	}
+	cg_controller = cgroup_get_controller(cg, "memory");
+	ret = cgroup_get_value_int64(cg_controller, "memory.memsw.max_usage_in_bytes", &max_usage);
+	if ( ret!=0 ) {
+		error(0,"get cgroup value - %s(%d)", cgroup_strerror(ret), ret);
+	}
+
+	fprintf(stderr, "Total memory used: %" PRId64 " kB\n", max_usage/1024);
+
+	cgroup_free(&cg);
+}
+
+void cgroup_create()
+{
+	int ret;
+	struct cgroup *cg;
+	struct cgroup_controller *cg_controller;
+
+	cg = cgroup_new_cgroup(cgroupname);
+	if (!cg) {
+		error(0,"cgroup_new_cgroup");
+	}
+
+	/* Set up the memory restrictions; these two options limit ram use
+	   and ram+swap use. They are the same so no swapping can occur */
+	cg_controller = cgroup_add_controller(cg, "memory");
+	cgroup_add_value_int64(cg_controller, "memory.limit_in_bytes", memsize);
+	cgroup_add_value_int64(cg_controller, "memory.memsw.limit_in_bytes", memsize);
+
+	/* Set up cpu restrictions; we pin the task to a specific set of cpus,
+	   based on the environment variable CPUSET. We also give it exclusive
+	   access to those cores, and set no limits on memory nodes */
+	if ( cpuset!=NULL && strlen(cpuset)>0 ) {
+		cg_controller = cgroup_add_controller(cg, "cpuset");
+		/* To make a cpuset exclusive, some additional setup outside of domjudge is
+		   required, so for now, we will leave this commented out. */
+		/* cgroup_add_value_int64(cg_controller, "cpuset.cpu_exclusive", 1); */
+		cgroup_add_value_string(cg_controller, "cpuset.mems", "0");
+		cgroup_add_value_string(cg_controller, "cpuset.cpus", cpuset);
+	} else {
+		fprintf(stderr, "CPUSET undefined\n");
+	}
+
+	/* Perform the actual creation of the cgroup */
+	ret = cgroup_create_cgroup(cg, 1);
+	if ( ret!=0 ) {
+		error(0,"creating cgroup - %s(%d)", cgroup_strerror(ret), ret);
+	}
+
+	cgroup_free(&cg);
+}
+
+void cgroup_attach()
+{
+	int ret;
+	struct cgroup *cg;
+
+	cg = cgroup_new_cgroup(cgroupname);
+	if (!cg) {
+		error(0,"cgroup_new_cgroup");
+	}
+	ret = cgroup_get_cgroup(cg);
+	if ( ret!=0 ) {
+		error(0,"get cgroup information - %s(%d)", cgroup_strerror(ret), ret);
+	}
+
+	/* Attach task to the cgroup */
+	ret = cgroup_attach_task(cg);
+	if ( ret!=0 ) {
+		error(0,"attach task to cgroup - %s(%d)", cgroup_strerror(ret), ret);
+	}
+
+	cgroup_free(&cg);
+}
+
+void cgroup_delete()
+{
+	int ret;
+	struct cgroup *cg;
+
+	cg = cgroup_new_cgroup(cgroupname);
+	if (!cg) {
+		error(0,"cgroup_new_cgroup");
+	}
+	ret = cgroup_get_cgroup(cg);
+	if ( ret!=0 ) {
+		error(0,"get cgroup information - %s(%d)", cgroup_strerror(ret), ret);
+	}
+	/* Clean up our cgroup */
+	ret = cgroup_delete_cgroup(cg, 1);
+	if ( ret!=0 ) {
+		error(0,"deleting cgroup - %s(%d)", cgroup_strerror(ret), ret);
+	}
+	cgroup_free(&cg);
+}
+#endif // USE_CGROUPS
+
 void terminate(int sig)
 {
 	struct sigaction sigact;
@@ -406,12 +538,14 @@ void setrestrictions()
 	}
 
 	if ( use_cputime ) {
-		rlim_t cputime_limit = (rlim_t)(cputime/1000000 + 1);
+		rlim_t cputime_limit = (rlim_t)cputime + 1;
 		verbose("setting CPU-time limit to %d seconds",(int)cputime_limit);
 		lim.rlim_cur = lim.rlim_max = cputime_limit;
 		setlim(CPU);
 	}
 
+	/* Memory limits may be handled by cgroups now */
+#ifndef USE_CGROUPS
 	if ( memsize!=RLIM_INFINITY ) {
 		verbose("setting memory limits to %d bytes",(int)memsize);
 		lim.rlim_cur = lim.rlim_max = memsize;
@@ -419,6 +553,13 @@ void setrestrictions()
 		setlim(DATA);
 		setlim(STACK);
 	}
+#else
+	/* Memory limits should be unlimited when using cgroups */
+	lim.rlim_cur = lim.rlim_max = RLIM_INFINITY;
+	setlim(AS);
+	setlim(DATA);
+	setlim(STACK);
+#endif
 
 	if ( filesize!=RLIM_INFINITY ) {
 		verbose("setting filesize limit to %d bytes",(int)filesize);
@@ -495,14 +636,15 @@ int main(int argc, char **argv)
 	fd_set readfds;
 	pid_t pid;
 	int   i, r, nfds;
+#ifdef USE_CGROUPS
+	int   ret;
+#endif
 	int   status;
 	int   exitcode;
 	char *valid_users;
 	char *ptr;
 	int   opt;
-	double runtime_d;
-	double cputime_d;
-	double timediff;
+	double timediff, tmpd;
 	size_t data_passed[3];
 	ssize_t nread, nwritten;
 
@@ -513,12 +655,12 @@ int main(int argc, char **argv)
 
 	/* Parse command-line options */
 	use_root = use_time = use_cputime = use_user = outputexit = outputtime = no_coredump = 0;
-	cputime = memsize = filesize = nproc = RLIM_INFINITY;
+	memsize = filesize = nproc = RLIM_INFINITY;
 	redir_stdout = redir_stderr = limit_streamsize = 0;
 	be_verbose = be_quiet = 0;
 	show_help = show_version = 0;
 	opterr = 0;
-	while ( (opt = getopt_long(argc,argv,"+r:u:g:t:C:m:f:p:co:e:s:E:T:vq",long_opts,(int *) 0))!=-1 ) {
+	while ( (opt = getopt_long(argc,argv,"+r:u:g:t:C:m:f:p:P:co:e:s:E:T:vq",long_opts,(int *) 0))!=-1 ) {
 		switch ( opt ) {
 		case 0:   /* long-only option */
 			break;
@@ -541,20 +683,17 @@ int main(int argc, char **argv)
 			break;
 		case 't': /* time option */
 			use_time = 1;
-			runtime_d = strtod(optarg,&ptr);
-			if ( errno || *ptr!='\0' ||
-			     runtime_d<=0 || runtime_d>=ULONG_MAX*1E-6 ) {
+			runtime = strtod(optarg,&ptr);
+			if ( errno || *ptr!='\0' || !finite(runtime) || runtime<=0 ) {
 				error(errno,"invalid runtime specified: `%s'",optarg);
 			}
-			runtime = (unsigned long)(runtime_d*1E6);
 			break;
 		case 'C': /* CPU time option */
 			use_cputime = 1;
-			cputime_d = strtod(optarg,&ptr);
-			if ( errno || *ptr!='\0' || !finite(cputime_d) || cputime_d<=0 ) {
+			cputime = strtod(optarg,&ptr);
+			if ( errno || *ptr!='\0' || !finite(cputime) || cputime<=0 ) {
 				error(errno,"invalid cputime specified: `%s'",optarg);
 			}
-			cputime = (int)(cputime_d*1E6);
 			break;
 		case 'm': /* memsize option */
 			memsize = (rlim_t) readoptarg("memory limit",1,LONG_MAX);
@@ -577,6 +716,13 @@ int main(int argc, char **argv)
 		case 'p': /* nproc option */
 			nproc = (rlim_t) readoptarg("process limit",1,LONG_MAX);
 			break;
+		case 'P': /* cpuset option */
+			#ifdef USE_CGROUPS
+				cpuset = optarg;
+			#else
+				error(0,"This option is only supported when compiled with cgroup support.");
+			#endif
+			break;
 		case 'c': /* no-core option */
 			no_coredump = 1;
 			break;
@@ -663,6 +809,24 @@ int main(int argc, char **argv)
 		error(errno,"installing signal handler");
 	}
 
+#ifdef USE_CGROUPS
+	/* Make libcgroup ready for use */
+	ret = cgroup_init();
+	if ( ret!=0 ) {
+		error(0,"libcgroup initialization failed: %s(%d)\n", cgroup_strerror(ret), ret);
+	}
+	/* Define the cgroup name that we will use */
+	cgroupname = (char*)malloc(256);
+	if ( cgroupname==NULL ) {
+		error(errno,"allocating memory for cgroupname");
+	}
+	/* Note: group names must have slashes! */
+	snprintf(cgroupname, 256, "/domjudge/dj_cgroup_%d/", getpid());
+
+	cgroup_create();
+
+	unshare(CLONE_FILES|CLONE_FS|CLONE_NEWIPC|CLONE_NEWNET|CLONE_NEWNS|CLONE_NEWUTS|CLONE_SYSVSEM);
+#endif
 	switch ( child_pid = fork() ) {
 	case -1: /* error */
 		error(errno,"cannot fork");
@@ -682,6 +846,11 @@ int main(int argc, char **argv)
 		   and all its children can be killed off with one signal. */
 		if ( setsid()==-1 ) error(errno,"setsid failed");
 
+#ifdef USE_CGROUPS
+		/* Put the child process in the cgroup */
+		cgroup_attach();
+#endif
+
 		/* Apply all restrictions for child process. */
 		setrestrictions();
 
@@ -752,13 +921,13 @@ int main(int argc, char **argv)
 			/* Trigger SIGALRM via setitimer:  */
 			itimer.it_interval.tv_sec  = 0;
 			itimer.it_interval.tv_usec = 0;
-			itimer.it_value.tv_sec  = runtime / 1000000;
-			itimer.it_value.tv_usec = runtime % 1000000;
+			itimer.it_value.tv_sec  = (int) runtime;
+			itimer.it_value.tv_usec = (int)(modf(runtime,&tmpd) * 1E6);
 
 			if ( setitimer(ITIMER_REAL,&itimer,NULL)!=0 ) {
 				error(errno,"setting timer");
 			}
-			verbose("using timelimit of %.3f seconds",runtime*1E-6);
+			verbose("using timelimit of %.3f seconds",runtime);
 		}
 
 		if ( times(&startticks)==(clock_t) -1 ) {
@@ -837,6 +1006,11 @@ int main(int argc, char **argv)
 			exitcode = WEXITSTATUS(status);
 		}
 
+#ifdef USE_CGROUPS
+		output_cgroup_stats();
+		cgroup_delete();
+#endif
+
 		/* Drop root before writing to output file(s). */
 		if ( setuid(getuid())!=0 ) error(errno,"dropping root privileges");
 
