@@ -79,6 +79,19 @@ if ( ! isset($_POST['cancel']) ) {
 	}
 }
 
+// If the form contained uploadable files, process these now.
+if ( isset($_FILES['data']) ) {
+	foreach($_FILES['data']['tmp_name'] as $id => $tmpnames) {
+		foreach($tmpnames as $field => $tmpname) {
+			if ( !empty ($tmpname) ) { 
+				checkFileUpload($_FILES['data']['error'][$id][$field]);
+				$itemdata = array($field => file_get_contents($tmpname));
+				$DB->q("UPDATE $t SET %S WHERE %S", $itemdata, $prikey);
+			}
+		}
+	}
+}
+
 // Throw the user back to the page he came from, if not available
 // to the overview for the edited data.
 if ( !empty($referrer) ) {
